@@ -1,4 +1,4 @@
-/***             analog 4.04             http://www.analog.cx/             ***/
+/***             analog 4.11             http://www.analog.cx/             ***/
 /*** This program is copyright (c) Stephen R. E. Turner 1995 - 2000 except as
  *** stated otherwise. Distribution, usage and modification of this program is
  *** subject to the conditions of the Licence which you should have received
@@ -23,7 +23,7 @@
 
 #define BLOCKSIZE (32768)
 /* Analog allocates memory in blocks of this many bytes. Also any logfile
-   lines longer than this will be rejected. */
+   lines this long or longer will be rejected. */
 
 #define MAX_CONFIGS (50)
 /* The maximum number of configuration files that are allowed. */
@@ -100,8 +100,8 @@
 /* Whether to convert %nm's from the top half of the character set in the
    search reports */
 
-#define REPORTORDER "xmWDdHh45uJoZSvitzcPrEIfsNnkKbBp"
-/* A string like "xmWDdHh45uJoZSvitzcrEIfsNnkKbBp" giving the order in which
+#define REPORTORDER "xmWDdHh45oZSuJkKfsNnBbpvcPztiEIr"
+/* A string like "xmWDdHh45oZSuJkKfsNnBbpvcPztiEIr" giving the order in which
    the reports will be output. You should include each of the above letters
    exactly once. See docs/output.html for which report has which code letter.*/
 
@@ -241,8 +241,8 @@
 #define FAILREFARGSFLOOR "10r"
 #define BROWFLOOR "-20r"
 #define FULLBROWFLOOR "-40r"
-#define OSFLOOR "1r"
-#define SUBOSFLOOR "1r"
+#define OSFLOOR "0r"
+#define SUBOSFLOOR "0r"
 #define VHOSTFLOOR "-20b"
 #define USERFLOOR "-100r"
 #define FAILUSERFLOOR "-20r"
@@ -309,16 +309,16 @@
 /* And one placed at the bottom of the page */
 
 #define OUTPUT (HTML)
-/* Default output type: ASCII, HTML, COMPUTER or CACHE */
+/* Default output type: HTML, PLAIN, ASCII, COMPUTER or CACHE */
 
 #ifndef LANGUAGE
 #define LANGUAGE (ENGLISH)
 /* Default language for the output page:
    ARMENIAN, BOSNIAN, CATALAN, SIMP_CHINESE (GB2312), TRAD_CHINESE (Big5),
    CZECH, DANISH, DUTCH, ENGLISH, US_ENGLISH, FINNISH, FRENCH, GERMAN, GREEK,
-   ICELANDIC, ITALIAN, JAPANESE, NORWEGIAN (Bokmal), NYNORSK, POLISH,
-   PORTUGUESE, BR_PORTUGUESE, RUSSIAN, SERBIAN, SLOVAK, SLOVENE, SPANISH,
-   SWEDISH, TURKISH or UKRAINIAN.
+   HUNGARIAN, ICELANDIC, ITALIAN, JAPANESE, KOREAN, NORWEGIAN (Bokmal),
+   NYNORSK, POLISH, PORTUGUESE, BR_PORTUGUESE, ROMANIAN, RUSSIAN, SERBIAN,
+   SLOVAK, SLOVENE, SPANISH, SWEDISH, TURKISH or UKRAINIAN.
    *
    NB The other languages that were available in version 3 have language
    files in the lang directory: but they are incomplete, and so can't be
@@ -333,7 +333,7 @@
 /* The default bar style for the displays -- 'a' to 'h' */
 
 #define HTMLPAGEWIDTH (65)
-#define ASCIIPAGEWIDTH (76)
+#define PLAINPAGEWIDTH (76)
 /* The width of the output for HTML and non-HTML */
 
 #define MINGRAPHWIDTH (15)
