@@ -1,4 +1,4 @@
-/***             analog 4.04             http://www.analog.cx/             ***/
+/***             analog 4.11             http://www.analog.cx/             ***/
 /*** This program is copyright (c) Stephen R. E. Turner 1995 - 2000 except as
  *** stated otherwise. Distribution, usage and modification of this program is
  *** subject to the conditions of the Licence which you should have received
@@ -14,7 +14,7 @@
 
 /*** OS specific things ***/
 
-#define VNUMBER "4.04"    /* the version number of this program */
+#define VNUMBER "4.11"    /* the version number of this program */
 
 #undef VERSION /* If no OS set, compilation will fail by not knowing VERSION */
 
@@ -36,7 +36,6 @@
 #define NOPIPES
 #define MACDIRENT
 #define NOOPEN
-#define NOREGEX
 #define LINE_PARSER
 #define USE_ZLIB
 #include <MacHeaders.h>
@@ -48,7 +47,6 @@
 
 #ifdef DOS
 #define NODNS
-#define NOREGEX
 #define DIRSEP '\\'
 #define EXTSEP "."
 #define VERSION VNUMBER"/DOS"
@@ -57,7 +55,6 @@
 #ifdef VMS
 #define NOPIPES
 #define VMSDIRENT
-#define NOREGEX
 #ifdef __VMS_VER
 #if (__VMS_VER < 70000000)
 #define NOGMTIME
@@ -89,7 +86,6 @@
 extern void Win32Init(void);
 extern void Win32Cleanup(void);
 #define WIN32DIRENT  /* can comment out if POSIX dirent present, eg Borland */
-#define NOREGEX
 #define FOPENR_BINARY
 #define POPENR_BINARY
 #define DIRSEP '\\'
@@ -98,7 +94,6 @@ extern void Win32Cleanup(void);
 #endif
 
 #ifdef OS2
-#define NOREGEX
 #define DIRSEP '\\'
 #define EXTSEP "."
 #define VERSION VNUMBER"/OS2"
@@ -108,7 +103,6 @@ extern void Win32Cleanup(void);
 #define NODNS
 #define NOPIPES
 #define NODIRENT
-#define NOREGEX
 #define DIRSEP '.'
 #define EXTSEP "/"
 #define VERSION VNUMBER"/RISCOS"
@@ -117,7 +111,6 @@ extern void Win32Cleanup(void);
 #ifdef NEXTSTEP
 #define DIRSEP '/'
 #define EXTSEP "."
-#define NOREGEX
 #ifndef S_IWUSR
 #define S_IWUSR 0000200 /* write permission, owner */
 #endif
@@ -156,6 +149,8 @@ extern void Win32Cleanup(void);
 
 /*** system dependent header files ***/
 
+#include "pcre.h"
+
 #ifdef WIN32
 #include <windows.h>
 #endif
@@ -251,17 +246,6 @@ extern void mac_free(void *ptr);
 #endif
 #endif
 
-#ifdef NOREGEX
-typedef ptrdiff_t regoff_t;
-typedef struct
-{
-  regoff_t rm_so;  /* Byte offset from string's start to substring's start.  */
-  regoff_t rm_eo;  /* Byte offset from string's start to substring's end.  */
-} regmatch_t;
-#else
-#include <regex.h>
-#endif
-
 /*** User-definable header file ***/
 
 #include "anlghead.h"
@@ -291,9 +275,7 @@ typedef struct
 #define ERRBUFMODE (_IOLBF)
 #endif
 #define FEW (2)
-#define REDIRECT (2)
 #define NEWLOGFMT (2)
-#define INFO (3)
 #define BADLOGFMT (3)
 #define CONTINUATION (-1)  /* used in warn() in utils.c */
 #define REGEX_INC (2)  /* next four used in (struct include).type */
@@ -301,6 +283,8 @@ typedef struct
 #define NORMAL_EXC (-1)
 #define REGEX_EXC (-2)
 #define BLOCK_EPSILON (30)
+#define MIN_SC (100)   /* see parsescode() in input.c */
+#define SC_NUMBER (600)
 #define IGNORE_CODE (1000)
 #define EPSILON (1e-8)
 #define MINS_IN_WEEK (10080)
@@ -313,6 +297,7 @@ typedef struct
 #define LAST_TIME (ULONG_MAX)
 #define WHITESPACE ((char)(-1))
 #define DOMLEVEL_NUMBER (677)  /* 26^2 + 1 */
+#define PMATCH_SIZE (30)  /* This allows PCRE to capture 9 subexpressions */
 #ifdef MAC_EVENTS
 #define MAC_IDLE_FREQ (200)
 #endif
@@ -338,6 +323,7 @@ typedef struct
 #define LOG_EXTENDED1 "#Fields:%x3"
 #define LOG_EXTENDED2 "#%j"
 #define LOG_MS_EXTENDED1 "#Fields:%x5"
+#define LOG_WEBSTAR_EXTENDED1 "#Fields:%x6"
 #define LOG_WEBSITE_NA \
   "%m/%d/%y %h:%n:%j\t%S\t%v\t%j\t%u\t%j\t%r\t%f\t%j\t%B\t%c\t%b\t%T"
 #define LOG_WEBSITE_INT \
@@ -351,17 +337,18 @@ typedef struct
 
 /* Types of item and other inputs: must have ITEMs in process_data() order:
    code, bytes, proctime, unixtime, am, date in some order (these are
-   INPUT_NUMBER): then rest in some order. */
+   INPUT_NUMBER, not to be confused with INP_NUMBER): then rest in some order.
+   NB I don't know if changing process_data() order would have side-effects. */
 /* See also item_type in globals.c */
 enum {ITEM_VHOST, ITEM_FILE, ITEM_USER, ITEM_REFERRER, ITEM_BROWSER, ITEM_HOST,
       INP_CODE, INP_BYTES, INP_PROCTIME, INP_UNIXTIME, INP_AM, INP_DATE,
-      INP_YEAR, INP_MONTH, INP_HOUR, INP_MIN, INP_QUERY, INP_NUMBER};
+      INP_YEAR, INP_MONTH, INP_HOUR, INP_MIN, INP_QUERY, INP_IP, INP_NUMBER};
 #define ITEM_NUMBER (INP_CODE)
 #define INPUT_NUMBER (INP_YEAR)  /* NB There is INP_NUMBER and INPUT_NUMBER */
 
 /* Types of report: must have date reports then GENSUM then other reports,
    finishing with derv(tree)reps, then SIZE, CODE, PROCTIME in some order. */
-/* See also repcodes and repname in globals.c */
+/* See also repcodes, anchorname and repname in globals.c */
 enum {REP_MONTH, REP_WEEK, REP_DAYREP, REP_DAYSUM, REP_HOURREP,	REP_HOURSUM,
       REP_QUARTER, REP_FIVE, REP_GENSUM, REP_REQ, REP_REDIR, REP_FAIL,
       REP_TYPE, REP_DIR, REP_HOST, REP_DOM, REP_REF, REP_REFSITE, REP_REDIRREF,
@@ -377,6 +364,9 @@ enum {REP_MONTH, REP_WEEK, REP_DAYREP, REP_DAYSUM, REP_HOURREP,	REP_HOURSUM,
 #define ARRAYREP_NUMBER (3)
 #define LAST_NORMALREP (REP_OS)
 
+/* Types of status code. Order insignificant. */
+enum {SUCCESS, FAILURE, REDIRECT, INFO, UNWANTED};
+
 /* Counts then dates collected about each item, and then other floor/sort
    criteria: order otherwise insignificant. See also methodname in globals.c */
 enum {REQUESTS, PAGES, REDIR, FAIL, SUCCDATE, REDIRDATE, FAILDATE, BYTES,
@@ -398,17 +388,18 @@ enum {COL_TITLE, COL_REQS, COL_PAGES, COL_BYTES, COL_PREQS, COL_PPAGES,
 /* Levels of DNS lookup */
 enum {DNS_NONE, DNS_READ, DNS_LOOKUP, DNS_WRITE};
 
-/* Output types: start with HTML then ASCII */
-enum {HTML, ASCII, COMPUTER, OUT_NONE};
+/* Output types: start with HTML then PLAIN */
+enum {HTML, PLAIN, ASCII, COMPUTER, OUT_NONE};
 
 /* days of the week */
 enum {SUNDAY, MONDAY, TUESDAY, WEDNESDAY, THURSDAY, FRIDAY, SATURDAY};
 
 /* languages; see also country in globals.c */
 enum {ARMENIAN, BOSNIAN, CATALAN, SIMP_CHINESE, TRAD_CHINESE, CZECH, DANISH,
-      DUTCH, ENGLISH, US_ENGLISH, FINNISH, FRENCH, GERMAN, GREEK, ICELANDIC,
-      ITALIAN, JAPANESE, NORWEGIAN, NYNORSK, POLISH, PORTUGUESE, BR_PORTUGUESE,
-      RUSSIAN, SERBIAN, SLOVAK, SLOVENE, SPANISH, SWEDISH, TURKISH, UKRAINIAN};
+      DUTCH, ENGLISH, US_ENGLISH, FINNISH, FRENCH, GERMAN, GREEK, HUNGARIAN,
+      ICELANDIC, ITALIAN, JAPANESE, KOREAN, NORWEGIAN, NYNORSK, POLISH,
+      PORTUGUESE, BR_PORTUGUESE, ROMANIAN, RUSSIAN, SERBIAN, SLOVAK, SLOVENE,
+      SPANISH, SWEDISH, TURKISH, UKRAINIAN};
 
 /* lngstrs; charset_ must be first */
 enum {charset_, weekbeg_, month_, day_, days_, hr_, minute_, minutes_, second_,
@@ -495,13 +486,13 @@ typedef struct aliasto {
 
 typedef struct alias {
   char *from;
-#ifndef NOREGEX
-  regex_t *pattern;
-#endif
+  pcre *pattern;
   AliasTo *to;
-  int nmatch;
+  logical isregex;
   struct alias *next;
 } Alias;
+/* For a normal pattern, nmatch is no. of *'s. For a regular expression, it's
+   -3 * (no. of capturing subexpressions): this is what pcre_exec needs. */
 
 typedef struct strpair {
   char *name, *data;
@@ -566,7 +557,7 @@ typedef struct mmlist {
 typedef struct {
   void *block_end, *curr_pos, *next_pos;
   Mmlist *first, *last;
-  choice alignment;
+  unsigned short alignment;
 } Memman;
 
 typedef struct daysdata {
@@ -623,9 +614,7 @@ typedef struct strlist {
 
 typedef struct include {
   char *name;
-#ifndef NOREGEX
-  regex_t *pattern;
-#endif
+  pcre *pattern;
   choice type;
   struct include *next;
 } Include;
@@ -661,7 +650,7 @@ typedef struct {
   logical rawbytes;
   logical norobots;
   choice weekbeginson;
-  unsigned int asciipagewidth, htmlpagewidth, pagewidth, mingraphwidth;
+  unsigned int plainpagewidth, htmlpagewidth, pagewidth, mingraphwidth;
   char sepchar, repsepchar, decpt, barstyle;
   char *compsep;
   Tree *tree[GENREP_NUMBER];
@@ -693,6 +682,8 @@ typedef struct {
   Arraydata *arraydata[ARRAYREP_NUMBER];
   Hashindex **gooditems, **baditems;
   Include *wanthead[ITEM_NUMBER], *ispagehead, *argshead, *refargshead;
+  choice code2type[SC_NUMBER];
+  logical succ304;
   Alias *aliashead[ITEM_NUMBER];
   Strpair *searchengines;
   Dateman dman;
@@ -729,7 +720,7 @@ typedef struct {
    unsigned long, but it's probably OK for our purposes */
 #endif
 #define STREQ(s, t) (strcmp(s, t) == 0)
-#define MATCHES(s, p) (matchq(s, (void *)p, FALSE, 0, NULL))
+#define MATCHES(s, p) (matchq(s, (void *)p, FALSE, NULL))
 #define DAYOFWEEK(d) ((choice)(((d) + 3) % 7))   /* by defns. in dates.c */
 #define IS_EMPTY_STRING(s) ((s)[0] == '\0')
 #define IS_STDIN(s) (STREQ((s), "-") || STREQ((s), "stdin"))
@@ -761,9 +752,6 @@ typedef struct {
 /* see ref to IS_REGEXT above; NB can't use >> with negative numbers */
 #define IS_REGEXT(type) ((type) / 2)  /* for includes */
 #define IS_INC(type) ((type) > 0)     /* for includes */
-#define ENCODE_NMATCH(n) (-(n) - 1)   /* configalias(), init2.c */
-#define DECODE_NMATCH(n) (-(n))       /* do_aliasx(), alias.c */
-#define IS_REGEXN(n) ((n) < 0)        /* for aliases */
 /* static char *s = NULL; static size_t len = 0; ENSURE_LEN(s, len, req_len) */
 #define ENSURE_LEN(s, l, n) if ((n) > (l)) { \
                               s = (char *)xrealloc((void *)s, n); \
@@ -794,7 +782,7 @@ typedef struct {
 #define CLREPTOGGLE(n) CLTOGGLE(op->outopts.repq[n])
 #define CLOUTSTYLE CLTOGGLE
 #define CLGOTOS CLTOGGLE
-/* these happen to work because ASCII == TRUE == ON and HTML == FALSE == OFF */
+/* these happen to work because PLAIN == TRUE == ON and HTML == FALSE == OFF */
 /* abbreviations for output */
 #define PRESTART() if (outstyle == HTML) fputs("<pre><tt>", outf);\
                    else if (outstyle == ASCII) fputc('\n', outf);
