@@ -1,4 +1,4 @@
-/***             analog 4.04             http://www.analog.cx/             ***/
+/***             analog 4.11             http://www.analog.cx/             ***/
 /*** This program is copyright (c) Stephen R. E. Turner 1995 - 2000 except as
  *** stated otherwise. Distribution, usage and modification of this program is
  *** subject to the conditions of the Licence which you should have received
@@ -31,10 +31,11 @@ extern logical do_aliasb(char *name);
 extern logical dnsresolve(char *name, choice level);
 void do_dns(char *name, char *alias, choice level);
 #endif
-extern void wildalias(char *name, AliasTo *to, regmatch_t pmatch[]);
+extern void wildalias(char *name, AliasTo *to, ptrdiff_t pmatch[]);
 extern choice prealias(Memman *mm, Memman *mmv, Hashentry *vhost, Memman *mmq,
   logical case_insensitive, char *prefix, size_t prefixlen, int pvpos,
   Include *argshead);
+extern void prealiasS(Memman *mm, Memman *mms);
 extern void reverseonename(Hashindex *p);
 extern void reversenames(Hashindex *ans);
 
@@ -103,7 +104,7 @@ extern void confline(Options *op, char *cmd, char *arg1, char *arg2, int rc);
 extern choice config(char *filename, Options *op);
 extern void settings(Options *op, int argc, char *argv[]);
 extern Inputformatlist *correctlogfmt(Filelist *lp, Include **wanthead,
-  Dateman *dman);
+  choice *code2type, Dateman *dman);
 extern void correct(Options *op);
 extern void finalinit(Options *op);
 extern choice strtoinfmt(Inputformat **ans, char *s, choice *count);
@@ -186,6 +187,10 @@ extern void configincd(void *opt, char *cmd, char *arg1, char *arg2, int rc);
 extern void configexcd(void *opt, char *cmd, char *arg1, char *arg2, int rc);
 extern void configincs(void *opt, char *cmd, char *arg1, char *arg2, int rc);
 extern void configexcs(void *opt, char *cmd, char *arg1, char *arg2, int rc);
+void configscinex(void *opt, char *cmd, char *arg1, char *arg2, int rc,
+  logical in);
+extern void configscinc(void *opt, char *cmd, char *arg1, char *arg2, int rc);
+extern void configscexc(void *opt, char *cmd, char *arg1, char *arg2, int rc);
 
 /* in input.c */
 extern FILE *my_lfopen(Filelist *p, char *logtype);
@@ -196,7 +201,7 @@ extern int my_lfclose(FILE *f, Filelist *p, Filelist *allc, Filelist *allp,
 extern int my_fclose(FILE *f, char *name, char *filetype);
 extern FILE *logfile_init(Filelist *logfilep);
 extern choice getmoredata(FILE *f, char *start, size_t length);
-extern choice getnextline(FILE *f, char *start);
+extern choice getnextline(FILE *f, char *start, size_t used);
 extern void shunt_data(FILE *f);
 extern int strtomonth(char *month);
 extern choice parsemonth(FILE *f, void *arg, char c);
@@ -205,7 +210,7 @@ extern choice parsenmonth(FILE *f, void *arg, char c);
 extern choice parseulong(FILE *f, void *arg, char c);
 extern choice parseproctime(FILE *f, void *arg, char c);
 extern choice parseuint2strict(FILE *f, void *arg, char c);
-extern choice parseuint3strict(FILE *f, void *arg, char c);
+extern choice parsescode(FILE *f, void *arg, char c);
 extern choice parseuint4strict(FILE *f, void *arg, char c);
 extern choice parseudint(FILE *f, void *arg, char c);
 extern choice parseyear(FILE *f, void *arg, char c);
@@ -220,6 +225,7 @@ extern choice parsestring(FILE *f, void *arg, char c);
 extern choice parsemsbrow(FILE *f, void *arg, char c);
 extern choice parseref(FILE *f, void *arg, char c);
 extern choice checkchar(FILE *f, void *arg, char c);
+extern void resolveconflicts(unsigned int n, char *delims, ...);
 extern choice parselogfmt(FILE *f, void *arg, char c);
 extern choice parsenextrecord(FILE *f, Inputformat *format);
 extern int parseconfline(char *s, char **cmd, char **arg1, char **arg2);
@@ -406,7 +412,7 @@ extern void colheads(FILE *outf, choice *cols, choice outstyle,
   char **lngstr);
 extern void printcols(FILE *outf, choice rep, choice *cols, choice outstyle,
   logical multibyte,
-  unsigned long reqs, unsigned long pages, double bys, long index,
+  unsigned long reqs, unsigned long pages, double bys, long index, int level,
   unsigned long totr, unsigned long totp, double totb, unsigned int width[],
   unsigned int bmult, char graphby, double unit, char barstyle, char markchar,
   char repsepchar, char decpt, char *compsep, logical name1st,
@@ -437,7 +443,7 @@ extern void checkreps(Outchoices *od, logical *repq, Dateman *dman,
 
 /* in process.c */
 extern void process_data(Filelist *logfilep, FILE *lf, Hashtable **hash,
-  Arraydata **arraydata, choice *count, Include **wanthead,
+  Arraydata **arraydata, choice *count, choice *code2type, Include **wanthead,
   Include *ispagehead, Alias **aliashead, Include *argshead,
   Include *refargshead, Dateman *dman, Tree **tree, Derv **derv,
   choice *alltrees, choice *alldervs, choice *lowmem, logical case_insensitive,
@@ -458,6 +464,7 @@ extern void report_logformat(FILE *outf, Inputformatlist *format,
   logical inwarn);
 extern void report_fromto(Dateman *dman);
 extern void report_want(Include *wanthead, char *type, char *indent);
+extern void report_scwant(choice *code2type);
 extern void report_alias(Alias *aliashead, char *type, char *indent);
 #ifndef NOPIPES
 extern void report_uncompress(void);
@@ -529,7 +536,7 @@ extern void nnextname(char **name, char **nameend, char *whole, Strpair *list);
 /* in utils.c */
 extern void my_exit(int status);
 extern void sighandler(int sig);
-extern int substrcasecmp(char *s, char *t);
+extern logical substrcaseeq(char *s, char *t);
 extern char *strtoupper(char *n);
 extern void strtolowerx(char *n);
 extern void toloweralias(Alias *ap, logical both);
@@ -547,8 +554,10 @@ extern unsigned int findbmult(double d);
 extern unsigned long arraymaxl(unsigned long *x, unsigned int size);
 extern double arraymaxd(double *x, unsigned int size);
 extern size_t arraymaxlen(char **s, unsigned int size, choice outstyle);
-void prettyprintf(FILE *outf, unsigned int pagewidth, char *s, va_list ap);
+extern void prettyprintf(FILE *outf, unsigned int pagewidth, char *s,
+  va_list ap);
 extern void mprintf(FILE *outf, unsigned int pagewidth, char *s, ...);
+extern void print_progversion(FILE *f);
 extern void warn(char c, choice domess, char *s, ...);
 extern void error(char *s, ...);
 extern void debug(char c, char *s, ...);
@@ -561,9 +570,8 @@ extern void freemm(Memman *m);
 extern void *memmove(void *dst0, const void *src0, size_t length);
 #endif
 extern logical wildmatch(char *s, char *p, char *whole, int nmatch,
-  regmatch_t *pmatch);
-extern logical matchq(char *s, void *p, logical is_regex, int nmatch,
-  regmatch_t *pmatch);
+  ptrdiff_t *pmatch);
+extern logical matchq(char *s, void *p, logical is_regex, ptrdiff_t *pmatch);
 extern logical genwildmatch(char *s, char *s2, char *p);
 extern logical headmatch(char *s, char *p);
 extern logical headcasematch(char *s, char *p);
