@@ -1,4 +1,4 @@
-/***             analog 4.04             http://www.analog.cx/             ***/
+/***             analog 4.11             http://www.analog.cx/             ***/
 /*** This program is copyright (c) Stephen R. E. Turner 1995 - 2000 except as
  *** stated otherwise. Distribution, usage and modification of this program is
  *** subject to the conditions of the Licence which you should have received
@@ -24,7 +24,7 @@ char *block_start, *block_end, *block_bell, *record_start, *pos;
 logical termchar[256];
 
 /* Memory allocation */
-Memman *xmemman, *amemman, mm[ITEM_NUMBER], mmq;
+Memman *xmemman, *amemman, mm[ITEM_NUMBER], mmq, mms;
 char *workspace;
 
 /* DNS lookups: externs so that no worries about whether to pass them around */
@@ -68,8 +68,6 @@ unsigned int errwidth = ERRLINELENGTH;
 unsigned long progressfreq = 0;
 unsigned int ppcol = 0;
 
-/* Report code letters in order of enum in anlghea3.h */
-char repcodes[] = "mWDdHh45xrEItiSofskKBvuJZNnbpzcP";
 /* Names for things, always in order of enums in anlghea3.h */
 char *item_type[] = {"virtual hosts", "files", "users", "referrers",
 		       "browsers", "hosts"};
@@ -78,10 +76,18 @@ char *methodname[] = {"requests", "pages", "", "", "date", "", "", "bytes",
 			"alphabetical", "random"};
 /* Filenames for each language, in order of enum in anlghea3.h */
 char *country[] = {"am", "ba", "cat", "cns", "cnt", "cz", "dk", "nl", "uk",
-		   "us", "fi", "fr", "de", "gr", "is", "it", "jp", "no", "no2",
-		   "pl", "pt", "br", "ru", "yu", "sk", "si", "es", "se", "tr",
-		   "ua"};
-
+		   "us", "fi", "fr", "de", "gr", "hu", "is", "it", "jp", "kr",
+		   "no", "no2", "pl", "pt", "br", "ro", "ru", "yu", "sk", "si",
+		   "es", "se", "tr", "ua"};
+/* Report code letters in order of enum in anlghea3.h */
+char repcodes[] = "mWDdHh45xrEItiSofskKBvuJZNnbpzcP";
+/* Names for the anchors within the report, in the same order */
+char *anchorname[] = {"Month", "Week", "DayRep", "DaySum", "HourRep",
+		      "HourSum", "Quarter", "Five", "GenSum", "Req", "Redir",
+		      "Fail", "Type", "Dir", "Host", "Dom", "Ref", "RefSite",
+		      "RedirRef", "FailRef", "BrowRep", "VHost", "User",
+		      "FailUser", "Org", "SearchQuery", "SearchWord",
+		      "BrowSum", "OS", "Size", "Code", "ProcTime"};
 /* These report names are in English, so not the same as in langfile */
 /* In order of enum in anlghea3.h */
 char *repname[] = {"Monthly Report", "Weekly Report", "Daily Report",
@@ -211,6 +217,7 @@ Inputfns inpfns[] = { /* first the ITEM fns in any order */
   {'F', ITEM_REFERRER, &parseref, &mm[ITEM_REFERRER]},
   {'A', ITEM_BROWSER, &parsemsbrow, &mm[ITEM_BROWSER]},
   {'q', INP_QUERY, &parsestring, &mmq},
+  {'s', INP_IP, &parsestring, &mms},
   {'n', INP_MIN, &parseuint2strict, (void *)&min},
   {'h', INP_HOUR, &parseuint2, (void *)&hr},
   {'a', INP_AM, &parseam, (void *)&am},
@@ -222,7 +229,7 @@ Inputfns inpfns[] = { /* first the ITEM fns in any order */
   {'U', INP_UNIXTIME, &parseulong, (void *)&unixtime},
   {'T', INP_PROCTIME, &parseulong, (void *)&proctime},
   {'t', INP_PROCTIME, &parseproctime, (void *)&proctime},
-  {'c', INP_CODE, &parseuint3strict, (void *)&code},
+  {'c', INP_CODE, &parsescode, (void *)&code},
   {'C', INP_CODE, &parsecode, (void *)&code},
   {'b', INP_BYTES, &parseudint, (void *)&bytes},
   {'w', UNSET, &parsespace, NULL},
@@ -432,6 +439,9 @@ Configfns cf[] = {
   {"USEREXCLUDE", &configexc, (void *)&(opts.wanthead[ITEM_USER])},
   {"VHOSTINCLUDE", &configinc, (void *)&(opts.wanthead[ITEM_VHOST])},
   {"VHOSTEXCLUDE", &configexc, (void *)&(opts.wanthead[ITEM_VHOST])},
+  {"STATUSINCLUDE", &configscinc, (void *)(opts.code2type)},
+  {"STATUSEXCLUDE", &configscexc, (void *)(opts.code2type)},
+  {"304ISSUCCESS", &configonoff, (void *)&(opts.succ304)},
   {"REQINCLUDE", &configinc, (void *)&(opts.outopts.wanthead[G(REP_REQ)])},
   {"REQEXCLUDE", &configexc, (void *)&(opts.outopts.wanthead[G(REP_REQ)])},
   {"REDIRINCLUDE", &configinc, (void *)&(opts.outopts.wanthead[G(REP_REDIR)])},
@@ -623,7 +633,8 @@ Configfns cf[] = {
   {"BROWLOWMEM", &configlowmem, (void *)&(opts.miscopts.lowmem[ITEM_BROWSER])},
   {"USERLOWMEM", &configlowmem, (void *)&(opts.miscopts.lowmem[ITEM_USER])},
   {"VHOSTLOWMEM", &configlowmem, (void *)&(opts.miscopts.lowmem[ITEM_VHOST])},
-  {"ASCIIPAGEWIDTH", &configuint, (void *)&(opts.outopts.asciipagewidth)},
+  {"ASCIIPAGEWIDTH", &configuint, (void *)&(opts.outopts.plainpagewidth)},
+  {"PLAINPAGEWIDTH", &configuint, (void *)&(opts.outopts.plainpagewidth)},
   {"HTMLPAGEWIDTH", &configuint, (void *)&(opts.outopts.htmlpagewidth)},
   {"MINGRAPHWIDTH", &configuint, (void *)&(opts.outopts.mingraphwidth)},
   {"ERRLINELENGTH", &configuint, (void *)&errwidth},
@@ -673,6 +684,9 @@ void globals(char *comname) {
   mmq.curr_pos = NULL;
   mmq.first = NULL;
   mmq.alignment = 1;
+  mms.curr_pos = NULL;
+  mms.first = NULL;
+  mms.alignment = 1;
   xmemman = (Memman *)xmalloc(sizeof(Memman));
   xmemman->curr_pos = NULL;
   xmemman->first = NULL;
@@ -1104,7 +1118,7 @@ void defaults(Options *op) {
   DEFAULTSTR(op->outopts.logo, LOGO);
   DEFAULTSTR(op->outopts.stylesheet, STYLESHEET);
   op->outopts.weekbeginson = WEEKBEGINSON;
-  op->outopts.asciipagewidth = ASCIIPAGEWIDTH;
+  op->outopts.plainpagewidth = PLAINPAGEWIDTH;
   op->outopts.htmlpagewidth = HTMLPAGEWIDTH;
   op->outopts.mingraphwidth = MINGRAPHWIDTH;
   op->outopts.sepchar = SEPCHAR;
@@ -1176,6 +1190,12 @@ void defaults(Options *op) {
     op->hash[i] = rehash(NULL, HASHSIZE, NULL);
     op->miscopts.lowmem[i] = 0;
   }
+  for (i = 0; i < MIN_SC; i++)
+    op->code2type[i] = UNWANTED;
+  for ( ; i < SC_NUMBER; i++)
+    op->code2type[i] = UNSET;
+  op->code2type[0] = UNSET;  /* code2type[0] used as a special marker in ... */
+  op->succ304 = TRUE;    /* ... configsinex(), correctlogfmt() & finalinit() */
   for (i = 0; i < ARRAYREP_NUMBER; i++) {
     if (i == REP_CODE - FIRST_ARRAYREP)
       thresh = codethresh;
@@ -1199,9 +1219,9 @@ void defaults(Options *op) {
   op->ispagehead = NULL;
   op->argshead = NULL;
   op->refargshead = NULL;
-  confline(op, "PAGEINCLUDE", "*.htm", NULL, -1);
+  confline(op, "PAGEINCLUDE", "REGEXPI:\\.asp$", NULL, -1);
   confline(op, "PAGEINCLUDE", "*/", NULL, -1);
-  confline(op, "PAGEINCLUDE", "*.html", NULL, -1);
+  confline(op, "PAGEINCLUDE", "REGEXPI:\\.html?$", NULL, -1);
   op->dman.fromstr = NULL;
   op->dman.tostr = NULL;
   op->dman.currdp = NULL;
