@@ -1,4 +1,4 @@
-/***             analog 4.04             http://www.analog.cx/             ***/
+/***             analog 4.11             http://www.analog.cx/             ***/
 /*** This program is copyright (c) Stephen R. E. Turner 1995 - 2000 except as
  *** stated otherwise. Distribution, usage and modification of this program is
  *** subject to the conditions of the Licence which you should have received
@@ -560,6 +560,8 @@ my_temp_malloc(size_t size)
 	return((void *)ret_ptr);
 	}
 
+#define MAC_LEAVE_FREE          (64*1024)
+
 static void *
 sub_mac_malloc(size_t size)
 {
@@ -567,11 +569,11 @@ sub_mac_malloc(size_t size)
 
 	size=(size+3)&~3;
 	if (size>=MY_BIG_SIZE) {
-		if (MaxBlock()<32*1024+size) return(my_temp_malloc(size));
+		if (MaxBlock()<=MAC_LEAVE_FREE+size) return(my_temp_malloc(size));
 		return(malloc(size));
 		}
 	if (size>my_cur_avail) {
-		if (MaxBlock()<32*1024+MY_BLOCK_SIZE) return(my_temp_malloc(size));
+		if (MaxBlock()<MAC_LEAVE_FREE+MY_BLOCK_SIZE) return(my_temp_malloc(size));
 		my_cur_block=malloc(MY_BLOCK_SIZE);
 		my_cur_avail=MY_BLOCK_SIZE;
 		}
