@@ -1,4 +1,4 @@
-/***             analog 4.04             http://www.analog.cx/             ***/
+/***             analog 4.11             http://www.analog.cx/             ***/
 /*** This program is copyright (c) Stephen R. E. Turner 1995 - 2000 except as
  *** stated otherwise. Distribution, usage and modification of this program is
  *** subject to the conditions of the Licence which you should have received
@@ -13,6 +13,8 @@ extern unsigned int *rep2lng;
 void pagetop(FILE *outf, Outchoices *od, Dateman *dman) {
   extern timecode_t starttimec;
   char *datestr;
+  double t0;
+  int t1, t2;
 
   if (od->cgi) {
     if (od->outstyle == HTML)
@@ -28,7 +30,8 @@ void pagetop(FILE *outf, Outchoices *od, Dateman *dman) {
 	    "content=\"text/html; charset=%s\">\n", od->lngstr[charset_]);
     if (od->norobots)
       fputs("<meta name=\"robots\" content=\"noindex,nofollow\">\n", outf);
-    fprintf(outf, "<meta name=\"GENERATOR\" content=\"analog%s\">\n", VERSION);
+    fprintf(outf, "<meta name=\"GENERATOR\" content=\"analog %s\">\n",
+	    VERSION);
     fprintf(outf, "<title>%s ", od->lngstr[webstatsfor_]);
     htmlfprintf(outf, od->outstyle, od->multibyte, od->hostname, &(od->html),
 		TRUE);
@@ -74,7 +77,7 @@ void pagetop(FILE *outf, Outchoices *od, Dateman *dman) {
     include_file(outf, od->outstyle, od->headerfile, 'h', od->pagewidth,
 		 &(od->html));
   if (od->outstyle == COMPUTER) {
-    fprintf(outf, "x%sVE%sanalog%s\n", od->compsep, od->compsep, VERSION);
+    fprintf(outf, "x%sVE%sanalog %s\n", od->compsep, od->compsep, VNUMBER);
     fprintf(outf, "x%sHN%s%s\n", od->compsep, od->compsep, od->hostname);
     if (!STREQ(od->hosturl, "none"))
       fprintf(outf, "x%sHU%s%s\n", od->compsep, od->compsep, od->hosturl);
@@ -123,8 +126,10 @@ void pagetop(FILE *outf, Outchoices *od, Dateman *dman) {
 			  dman->lasttime / 1440, (dman->lasttime % 1440) / 60,
 			  dman->lasttime % 60, 0, 0, od->monthname,
 			  od->dayname, 0, 0, NULL, FALSE));
-      mprintf(outf, od->pagewidth, " (%.1f %s).",
-	      (dman->lasttime - dman->firsttime) / 1440.0,
+      t0 = (dman->lasttime - dman->firsttime) / 1440.0 + 0.005;
+      t1 = (int)t0;
+      t2 = (int)(100 * (t0 - (double)t1));
+      mprintf(outf, od->pagewidth, " (%d%c%02d %s).", t1, od->decpt, t2,
 	      od->lngstr[days_]);
       mprintf(outf, 0, NULL);
     }
@@ -140,10 +145,10 @@ void pagebot(FILE *outf, Outchoices *od) {
   long secs;
 
   if (od->outstyle == HTML)
-    fprintf(outf, "<i>%s <a HREF=\"%s\">analog%s</a>.\n", od->lngstr[credit_],
-	    ANALOGURL, VERSION);
+    fprintf(outf, "<i>%s <a HREF=\"%s\">analog %s</a>.\n", od->lngstr[credit_],
+	    ANALOGURL, VNUMBER);
   else if (od->outstyle == ASCII)
-    fprintf(outf, "%s analog%s.\n", od->lngstr[credit_], VERSION);
+    fprintf(outf, "%s analog %s.\n", od->lngstr[credit_], VNUMBER);
   if (od->runtime && (od->outstyle == HTML || od->outstyle == ASCII)) {
     if (od->outstyle == HTML)
       fprintf(outf, "<br><b>%s:</b> ", od->lngstr[runtime_]);
@@ -178,12 +183,13 @@ void pagebot(FILE *outf, Outchoices *od) {
     fputs("alt=\"HTML 2.0 Conformant!\"></a>\n", outf);
   }
   if (od->outstyle == HTML)
-    fputs("\n</body>\n</html>\n", outf);
+    fputs("</body>\n</html>\n", outf);
 }
 
 /* Print "goto"s. Assume outstyle == HTML and want-gotos already tested. */
 void gotos(FILE *outf, int code, logical *repq, char **lngstr,
 	   choice *reporder) {
+  extern char *anchorname[];
   int i;
 
   fprintf(outf, "<p>(<b>%s</b>", lngstr[goto_]);
@@ -192,8 +198,8 @@ void gotos(FILE *outf, int code, logical *repq, char **lngstr,
     if (reporder[i] == code)
       fprintf(outf, "%s %s", lngstr[colon_], lngstr[rep2lng[reporder[i]]]);
     else if (repq[reporder[i]])
-      fprintf(outf, "%s <a HREF=\"#Rep%d\">%s</a>", lngstr[colon_],
-	      reporder[i], lngstr[rep2lng[reporder[i]]]);
+      fprintf(outf, "%s <a HREF=\"#%s\">%s</a>", lngstr[colon_],
+	      anchorname[reporder[i]], lngstr[rep2lng[reporder[i]]]);
   }
   fputs(")\n", outf);
 }
@@ -201,9 +207,11 @@ void gotos(FILE *outf, int code, logical *repq, char **lngstr,
 void report_title(FILE *outf, char *name, int code, choice gotoq,
 		  logical *repq, char **lngstr, choice *reporder,
 		  choice outstyle) {
+  extern char *anchorname[];
 
   if (outstyle == HTML) {
-    fprintf(outf, "<h2><a NAME=\"Rep%d\">%s</a></h2>\n", code, name);
+    fprintf(outf, "<h2><a NAME=\"%s\">%s</a></h2>\n", anchorname[code],
+	    name);
     if (gotoq == TRUE)
       gotos(outf, code, repq, lngstr, reporder);
   }
@@ -1086,8 +1094,8 @@ void colheads(FILE *outf, choice *cols, choice outstyle, unsigned int width[],
 }
 
 void printcols(FILE *outf, choice rep, choice *cols, choice outstyle,
-	       logical multibyte,
-	       unsigned long reqs, unsigned long pages, double bys, long index,
+	       logical multibyte, unsigned long reqs, unsigned long pages,
+	       double bys, long index, int level,
 	       unsigned long totr, unsigned long totp, double totb,
 	       unsigned int width[], unsigned int bmult, char graphby,
 	       double unit, char barstyle, char markchar, char repsepchar,
@@ -1101,6 +1109,8 @@ void printcols(FILE *outf, choice rep, choice *cols, choice outstyle,
 	       unsigned int daylen, unsigned int plainmonthlen,
 	       unsigned int plaindaylen) {
   /* NB multibyte only used if name1st == FALSE; so not needed for time reps */
+  /* 'level' is -1 for time reports, 0 for other non-hierarchical reports
+     (although both are currently ignored). */
   extern char *workspace;   /* see top of alias.c */
   extern char repcodes[];
   static char *datestr = NULL;
@@ -1109,6 +1119,8 @@ void printcols(FILE *outf, choice rep, choice *cols, choice outstyle,
 
   if (outstyle == COMPUTER) {
     fprintf(outf, "%c%s", repcodes[rep], compsep);
+    if (level >= 1)
+      putc('l', outf);
     for (c = 0; cols[c] != COL_NUMBER; c++) {
       switch(cols[c]) {
       case COL_REQS:
@@ -1155,6 +1167,8 @@ void printcols(FILE *outf, choice rep, choice *cols, choice outstyle,
     fprintf(outf, "%s: ", name);
   }
 
+  if (outstyle == COMPUTER && level >= 1)
+    fprintf(outf, "%d%s", level, compsep);
   for (c = 0; cols[c] != COL_NUMBER; c++) {
     switch(cols[c]) {
     case COL_REQS:
