@@ -1,4 +1,4 @@
-/***             analog 4.04             http://www.analog.cx/             ***/
+/***             analog 4.11             http://www.analog.cx/             ***/
 /*** This program is copyright (c) Stephen R. E. Turner 1995 - 2000 except as
  *** stated otherwise. Distribution, usage and modification of this program is
  *** subject to the conditions of the Licence which you should have received
@@ -43,6 +43,7 @@ void report_vbles(Options *op) {
 	   stz);
   for (i = 0; i < ITEM_NUMBER; i++)
     report_want(op->wanthead[i], item_type[i], "");
+  report_scwant(op->code2type);
   report_want(op->ispagehead, "as pages", "");
   report_want(op->argshead, "search arguments", "");
   report_want(op->refargshead, "referrer search arguments", "");
@@ -73,11 +74,6 @@ void report_compile(void) {
 #else
   printf("  DNS lookups included\n");
 #endif
-#ifdef NOREGEX
-  printf("  No regular expressions\n");
-#else
-  printf("  Regular expressions included\n");
-#endif
 #ifdef NODIRENT
   printf("  No dirent [so no wildcards in logfile names]\n");
 #else
@@ -280,6 +276,37 @@ void report_want(Include *wanthead, char *type, char *indent) {
   }
 }
 
+void report_scwant(choice *code2type) {
+  int i, j;
+  logical f;
+
+  for (i = MIN_SC, f = FALSE; i < SC_NUMBER && !f; i++) {
+    if (code2type[i] == UNWANTED)
+      f = TRUE;
+  }
+  if (f) {
+    printf("Including only the following status codes:\n  ");
+    for (i = MIN_SC, j = 0; i <= SC_NUMBER; i++) {
+      if (j != 0 && (i == SC_NUMBER || code2type[i] == UNWANTED)) {
+	if (f)  /* reusing f for first printing */
+	  f = FALSE;
+	else
+	  printf(",");
+	if (i == j + 1)
+	  printf("%d", j);
+	else
+	  printf("%d-%d", j, i - 1);
+	j = 0;
+      }
+      else if (j == 0 && i != SC_NUMBER && code2type[i] != UNWANTED)
+	j = i;
+    }
+    if (f)
+      printf("none");
+    printf("\n");
+  }
+}
+
 void report_alias(Alias *aliashead, char *type, char *indent) {
   static char *pre0 = "";
   static char *pre1 = "REGEXP:";
@@ -294,7 +321,7 @@ void report_alias(Alias *aliashead, char *type, char *indent) {
     for (ap = aliashead; ap != NULL; TO_NEXT(ap)) {
       from = ap->from;
       pre = pre0;
-      if (IS_REGEXN(ap->nmatch)) {
+      if (ap->isregex) {
 	if (from[6] == ':') {
 	  pre = pre1;
 	  from += 7;
@@ -308,7 +335,7 @@ void report_alias(Alias *aliashead, char *type, char *indent) {
       for (tp = ap->to; tp != NULL; TO_NEXT(tp)) {
 	printf("%s", tp->string);
 	if (tp->after >= 0)
-	  printf("$%d", (tp->after) + !IS_REGEXN(ap->nmatch));
+	  printf("$%d", (tp->after) / 2 + !(ap->isregex));
 	/* internal representation for !regex is one out */
       }
       putchar('\n');
@@ -419,6 +446,8 @@ void report_outopts(Outchoices *od) {
     }
     else if (od->outstyle == ASCII)
       printf("ASCII\n");
+    else if (od->outstyle == PLAIN)
+      printf("PLAIN\n");
     else {  /* OUT_NONE is reported above */
       printf("COMPUTER\n");
       printf("The field separator is \"%s\"\n", od->compsep);
