@@ -1,4 +1,4 @@
-/***             analog 4.04             http://www.analog.cx/             ***/
+/***             analog 4.11             http://www.analog.cx/             ***/
 /*** This program is copyright (c) Stephen R. E. Turner 1995 - 2000 except as
  *** stated otherwise. Distribution, usage and modification of this program is
  *** subject to the conditions of the Licence which you should have received
@@ -76,21 +76,21 @@ void sighandler(int sig) {
   my_exit(EXIT_SIGNAL);
 }
 
-int substrcasecmp(char *s, char *t) {
+logical substrcaseeq(char *s, char *t) {
   /* match the beginning of s case insensitively against lower case t */
-  /* (but only return 0 (equal) or 1 (!equal), not any alphabetical order) */
   for ( ; TOLOWER(*s) == *t && *t != '\0'; s++, t++)
     ;       /* *s != '\0' covered by above two cases */
   if (*t == '\0')
-    return(0);
+    return(TRUE);
   else
-    return(1);
+    return(FALSE);
 }
 
 char *strtoupper(char *n) {
+  /* convert a string to upper case UP TO the first line break */
   char *c;
-  for (c = n; *c != '\0'; c++)
-    if (ISLOWER(*c))  /* this is needed to stop const char *n crashing */
+  for (c = n; *c != '\0' && *c != '\n' && *c != '\r'; c++)
+    if (ISLOWER(*c))   /* needed to stop (known u.c.) const char *n crashing */
       *c = TOUPPER(*c);
   return(n);
 }
@@ -102,15 +102,16 @@ void strtolowerx(char *n) {  /* only used in next 2 fns. */
 
 void toloweralias(Alias *ap, logical both) {
   AliasTo *tp;
+  char *errstr;
+  int erroffset;
 
   for ( ; ap != NULL; TO_NEXT(ap)) {
-#ifndef NOREGEX
     if (headcasematch(ap->from, "REGEXP:"))
-      regcomp(ap->pattern, ap->from + 7, REG_EXTENDED | REG_ICASE);
+      ap->pattern = pcre_compile(ap->from + 7, PCRE_CASELESS | PCRE_DOTALL,
+				 (const char **)(&errstr), &erroffset, NULL);
     /* if original regcomp succeeded, this one should too */
     else if (!headcasematch(ap->from, "REGEXPI:"))
       /* nothing to do for REGEXPI: */
-#endif
       strtolowerx(ap->from);
     if (both) {
       for (tp = ap->to; tp != NULL; TO_NEXT(tp))
@@ -121,13 +122,14 @@ void toloweralias(Alias *ap, logical both) {
 
 void tolowerinc(Include *ip) {
   /* see comments in previous function */
+  char *errstr;
+  int erroffset;
+
   for ( ; ip != NULL; TO_NEXT(ip)) {
-#ifndef NOREGEX
     if (headcasematch(ip->name, "REGEXP:"))
-      regcomp(ip->pattern, ip->name + 7,
-	      REG_EXTENDED | REG_ICASE | REG_NOSUB);
+      ip->pattern = pcre_compile(ip->name + 7, PCRE_CASELESS | PCRE_DOTALL,
+				 (const char **)(&errstr), &erroffset, NULL);
     else if (!headcasematch(ip->name, "REGEXPI:"))
-#endif
       strtolowerx((char *)(ip->name));
   }
 }
@@ -282,6 +284,16 @@ void mprintf(FILE *outf, unsigned int pagewidth, char *s, ...) {
   va_end(ap);
 }
 
+void print_progversion(FILE *f) {
+  extern char *commandname;
+  static logical done = FALSE;
+
+  if (!done) {
+    fprintf(f, "%s: analog version %s\n", commandname, VERSION);
+    done = TRUE;
+  }
+}
+
 void warn(char c, choice domess, char *s, ...) {
   extern FILE *errfile;
   extern char *warn_args;
@@ -294,6 +306,7 @@ void warn(char c, choice domess, char *s, ...) {
   if (strchr(warn_args, c) != NULL) {
     if (errwidth == 0)
       errwidth = UINT_MAX;
+    print_progversion(errfile);
     va_start(ap, s);
     if (domess == CONTINUATION)
       mprintf(errfile, errwidth, "%s: ...cont..: ", commandname, c);
@@ -321,6 +334,7 @@ void error(char *s, ...) {
 
   if (errwidth == 0)
     errwidth = UINT_MAX;
+  print_progversion(errfile);
   va_start(ap, s);
   mprintf(errfile, errwidth, "%s: Fatal error: ", commandname);
   prettyprintf(errfile, errwidth, s, ap);
@@ -350,6 +364,7 @@ void debug(char c, char *s, ...) {
   va_list ap;
 
   if (strchr(debug_args, c) != NULL) {
+    print_progversion(errfile);
     va_start(ap, s);
     fprintf(errfile, "%c: ", c);
     vfprintf(errfile, s, ap);
@@ -561,12 +576,12 @@ void *memmove(void *dst0, const void *src0, size_t length) {
 
 #define SETPMATCH(ptr1, ptr2) \
   if (nmatch > 0) { \
-    pmatch->rm_so = ptr1 - whole; \
-    pmatch->rm_eo = ptr2 - whole; \
+    pmatch[0] = ptr1 - whole; \
+    pmatch[1] = ptr2 - whole; \
   }
 
 logical wildmatch(char *s, char *p, char *whole, int nmatch,
-		  regmatch_t *pmatch) {  /* match string s against pattern p */
+		  ptrdiff_t *pmatch) {  /* match string s against pattern p */
   char *ss, *pp;
 
   /* First match head portions */
@@ -611,7 +626,7 @@ logical wildmatch(char *s, char *p, char *whole, int nmatch,
      (p was incremented above). We choose to match tail portion afresh each
      time (cheap), rather than insert null bytes into (poss. const) s and p */
   for (ss = s; *ss != '\0'; ss++) {   /* reuse ss */
-    if (wildmatch(ss, p, whole, nmatch - 1, pmatch + 1)) {
+    if (wildmatch(ss, p, whole, nmatch - 2, pmatch + 2)) {
       SETPMATCH(s, ss);
       return(TRUE);
     }
@@ -619,14 +634,12 @@ logical wildmatch(char *s, char *p, char *whole, int nmatch,
   return(FALSE);
 }
 
-logical matchq(char *s, void *p, logical is_regex, int nmatch,
-	       regmatch_t *pmatch) {
-#ifndef NOREGEX
+logical matchq(char *s, void *p, logical is_regex, ptrdiff_t *pmatch) {
   if (is_regex)
-    return(regexec((regex_t *)p, s, (size_t)nmatch, pmatch, 0) == 0);
+    return(pcre_exec((pcre *)p, NULL, s, (int)strlen(s), 0, 0, (int *)pmatch,
+		     (pmatch == NULL)?0:PMATCH_SIZE) >= 0);
   else
-#endif
-    return(wildmatch(s, (char *)p, s, nmatch, pmatch));
+    return(wildmatch(s, (char *)p, s, (pmatch == NULL)?0:PMATCH_SIZE, pmatch));
 }
 
 logical genwildmatch(char *s, char *s2, char *p) {
@@ -699,12 +712,8 @@ logical included(char *name, logical ispage, Include *listhead) {
 	return(IS_INC(lp->type));
     }
     else if (matchq(name,
-#ifdef NOREGEX
-		    (void *)(lp->name),
-#else
 		    is_regex?((void *)(lp->pattern)):((void *)(lp->name)),
-#endif
-		    is_regex, 0, NULL))
+		    is_regex, NULL))
       return(IS_INC(lp->type));
     lastlp = lp;
   }
@@ -717,7 +726,9 @@ logical pageq(char *name, Include *ispagehead, choice type) {
   char *c;
   logical ans;
 
-  if (type != ITEM_FILE && type != ITEM_REFERRER)
+  if (type == ITEM_REFERRER)
+    return(TRUE);
+  if (type != ITEM_FILE)
     return(FALSE);
 
   if ((c = strchr(name, '?')) != NULL) {
